@@ -24,16 +24,31 @@
 
 #include "calctool.h"
 
-void initialise();
-void localize_number(char *, const char *, int);
-char *make_fixed(int *, char *, int, int, int);
-char *make_number(int *, int, int);
-void clear_display(int);
-void MPstr_to_num(char *, enum base_type, int *);
+void display_init(void);
+
+void display_reset(void);
+void localize_expression(char *, const char *, int, int *);
+void display_clear(int);
 void paren_disp(int);
-void refresh_display(int);
-void show_display(int *);
-void process_item(struct button *, int);
+void display_refresh(int);
+void display_set_number(int *);
+void display_set_string(char *);
+void display_set_error(const char *);
+
+void display_clear_stack(void);
+void display_push(void);
+void display_pop(void);
+void display_unpop(void);
+
+int display_insert(const char *, int);
+int display_backspace(int cursor);
+int display_delete(int);
+int display_surround(const char *, const char *, int);
+
+gboolean display_is_empty(void);
 gboolean display_is_result(void);
+gboolean display_is_usable_number(int *);
+
+int display_solve(int *);
 
 #endif /* DISPLAY_H */
